from .download import download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_file(xz_url + "ocean.blend",down_path)

bl_info = {
    "name": "Physical Open Waters",
    "author": "Physical Addons",
    "version": (1, 1, 1),
    "blender": (3, 6, 0),
    "description": "More free stuff for Blender: https://t.me/BlenderUniverse",
    "location": "World > Atmosphere",
    "wiki_url": "https://www.physicaladdons.com/pow/",
    "support": "COMMUNITY",
    "category": "Lighting",
    "tracker_url": "https://discord.gg/wvzPVzj9Vr"
    }
__version__  = bl_info["version"]
import bpy
from bpy.props import (
    PointerProperty,
)
from bpy.app.handlers import persistent

from . properties import (
    POW_Preset,
    POW_Settings,

    #POW_Ocean,
    register_pow_property_groups,
    update_preset_versions,
)

from . ui import *

from . ocean import (
    POW_OT_AddOcean,
    POW_OT_RemoveOcean,

    setup_default_presets,
)

from . preferences import *


# All the classes that need to be registered in Blender for the addon to work.
# This includes UI classes, PropertyGroups and operators.
REGISTER_CLASSES = (
    # Prop classes
    POW_Preset,
    POW_Settings,
    #POW_Ocean,
    # Operators
    POW_OT_AddOcean,
    POW_OT_RemoveOcean,
    POW_OT_OpenPresetsFolder,
    POW_OT_DeleteCurrentPreset,
	POW_OT_CreateNewPreset,
	POW_OT_SaveCurrentPreset,
    POW_OT_ApplyUnderwaterAll,
    POW_OT_ApplyUnderwaterSelected,
    POW_OT_RemoveUnderwaterAll,
    POW_OT_RemoveUnderwaterSelected,
    POW_OT_AddIntersection,
    POW_OT_RemoveIntersection,
    # UI classes
    POW_PT_OceanSettingsTB,
    POW_PT_MeshSettingsSettingsTB,
    POW_PT_MaterialSettingsTB,
    POW_PT_CameraSettingsTB,
    POW_PT_WavesSettingsTB,
    POW_PT_CapillarySettingsTB,
    POW_PT_WakeSettingsTB,
    POW_PT_UnderwaterSettingsTB,
    POW_PT_FooterTB,
    POW_UL_PresetList,
    # Preferences
    POW_MT_AddonPreferences,
)


@persistent
def depsgraph_update_handler(scene, depsgraph):
    world = scene.world

    selected_materiables = 0
    if len(bpy.context.selected_objects) > 0:
        for object in bpy.context.selected_objects:
            if hasattr(object.data, "materials") and len(object.data.materials) > 0:
                selected_materiables += 1

    if selected_materiables <= 0:
        world.pow_settings["submerged_shader"] = False
        world.pow_settings["submerged_advanced_intersection"] = False
    else:
        # Ensure correct value for the checkbox
        world.pow_settings["submerged_shader"] = True
        for object in bpy.context.selected_objects:
            if hasattr(object.data, "materials"):
                for material in object.data.materials:
                    if not "POW_UnderwaterGroups" in material and not "POW_UnderwaterIgnore" in material:
                        world.pow_settings["submerged_shader"] = False
                        break
        # TODO Check performance impact of all this ^
        
        world.pow_settings["submerged_advanced_intersection"] = True
        for object in bpy.context.selected_objects:
            intersection_node = bpy.data.node_groups.get('POW_Get_Intersection')
            if not intersection_node:
                # Node not present in data, no object is using it
                world.pow_settings["submerged_advanced_intersection"] = False
                break

            if hasattr(object, "modifiers"):
                has_node = False
                for mod in object.modifiers:
                    if mod.type == 'NODES':
                        if mod.node_group == intersection_node:
                            has_node = True
                            break
                if not has_node:
                    world.pow_settings["submerged_advanced_intersection"] = False
                    break


def register():
    # Register all classes
    for cls in REGISTER_CLASSES:
        bpy.utils.register_class(cls)
    
    bpy.types.World.pow_settings = PointerProperty(type = POW_Settings)
    register_pow_property_groups(bpy.context, {}, first_time = True)
    #bpy.types.Object.pow_ocean = PointerProperty(type = POW_Ocean)

    # Install default presets in addon's preset folder after addon has been installed
    setup_default_presets()
    update_preset_versions(bpy.context)

    # Handlers
    bpy.app.handlers.depsgraph_update_post.append(depsgraph_update_handler)


def unregister():
    prefs = bpy.context.preferences.addons[__package__].preferences
    if prefs.animatable_props:
        bpy.app.handlers.frame_change_post.remove(frame_change_handler)

    # Unregister all classes
    for cls in REGISTER_CLASSES:
        bpy.utils.unregister_class(cls)
    
    del bpy.types.World.pow_settings
    del bpy.types.Object.pow_ocean

    # Handlers
    bpy.app.handlers.depsgraph_update_post.remove(depsgraph_update_handler)


if __name__ == "__main__":
    register()