import bpy

from mathutils import *
from math import *


#GLOBALS

M_PI = 3.14159265358979323846
num_waves = 16 #NUMBER OF WAVES
G = 9.810 #GRAVITY


#FUNCTIONS

def fract(x):
    return x - floor(x)

def nrand(n):
    return fract(sin(n.dot( Vector((12.9898, 78.233)))) * 43758.5453)

def n4rand( n ):
    nrnd0 = nrand( n + Vector((0.07,0.07)) )
    nrnd1 = nrand( n + Vector((0.11,0.11)) )	
    nrnd2 = nrand( n + Vector((0.13,0.13)) )
    nrnd3 = nrand( n + Vector((0.17,0.17)) )
    return (nrnd0+nrnd1+nrnd2+nrnd3) / 4.0


#WAVE CALCULATION

def update_waves(self, context):
    world = context.scene.world
    ocean = world.pow_settings.ocean
    
    wave_calculation(ocean, ocean.pow_ocean)


def wave_calculation(object, settings):
    #Wave frequency distribution
    min = log(settings.lambda_min) / log(settings.wave_distribution_log)
    max = log(settings.lambda_max) / log(settings.wave_distribution_log)

    #object = bpy.context.active_object

    for i in range(num_waves):
        
        x = (i) / (num_waves - 1)

        klambda = pow(2.0, (1.0 - x) * min + x * max) #lambda is a reserved function in python
        knorm = 2.0 * M_PI / klambda
        omega = sqrt(G * knorm)
        omega0 = G / settings.wind_speed
        
        step = (max-min) / float(num_waves - 1)
        
        amplitude = 0.0
        ktheta = (n4rand(Vector((float(i), float(i) + 0.25))) * 2.0 - 1.0) * settings.wave_dispersion
        ktheta *= 1.0 / (1.0 + 40.0 * pow(omega0 / omega, 4.0))
        
        amplitude = (8.1e-3*G*G)/pow(omega,5.0)*exp(-0.74*pow(omega0/omega,4.0))
        amplitude *= 0.5*sqrt(2.0*M_PI*G/klambda)*float(num_waves)*step
        amplitude = settings.height_max * (1.0/num_waves) * sqrt(amplitude)
        
        if (amplitude > 1.0 / knorm):
            amplitude = 1.0 / knorm
        if (amplitude < -1.0 / knorm):
            amplitude = 1.0 / knorm

        dispersion = 2.0 * M_PI / klambda
        
        wavedir = settings.wave_angle

        object["wave" + str(i + 1)] = Vector((cos(ktheta+wavedir), sin(ktheta+wavedir), dispersion,amplitude))

        #object.data.attributes.new(name= str(i), type='FLOAT_COLOR', domain='POINT')

        val = (cos(ktheta+wavedir),sin(ktheta+wavedir),dispersion,amplitude,cos(ktheta+wavedir),sin(ktheta+wavedir),dispersion,amplitude,cos(ktheta+wavedir),sin(ktheta+wavedir),dispersion,amplitude,cos(ktheta+wavedir),sin(ktheta+wavedir),dispersion,amplitude)
        object.data.attributes[str(i)].data.foreach_set('color', val)
        
    object["HeightScale"] = settings.height_max

    bpy.context.scene.world["ocean"] = object # TODO this is temporary, make the nodes use `pow_settings.ocean` instead.

    # Force update for values to work in shaders
    for obj in bpy.context.view_layer.objects.selected:
        obj.select_set(state = True)
        obj.cache_release()